/**********************************************************************
  *                                                                    *
  * Copyright (c) 2005 , LCG ROOT MathLib Team                         *
  *                                                                    *
  *                                                                    *
  **********************************************************************/

// Header file for class AxisAngle
// 
// Created by: Lorenzo Moneta  at Wed May 11 10:37:10 2005
// 
// Last update: Wed May 11 10:37:10 2005
// 
#ifndef ROOT_MATH_AXISANGLE
#define ROOT_MATH_AXISANGLE 1

#include "GenVector/Rotation3DMatrix.h"
#include "GenVector/EulerAngles.h"
#include "GenVector/DisplacementVector3D.h"



namespace ROOT { 

  namespace Math { 


  /** 
     AxisAngle class describing a rotation as a direction axis and an angle of rotation around that axis 
  */ 
template <typename T> 
class AxisAngle {

public: 

  typedef T Scalar;

  /**
     definition of vecor axis 
   */
  typedef DisplacementVector3D<Cartesian3D<T> > Axis;


  /** 
     Default constructor (axis is z and angle is zero)
  */ 
  AxisAngle() : fAngle(0), fAxis(0,0,1) {} 

  /** 
     Construct from a rotation matrix
  */ 
  AxisAngle(const Rotation3DMatrix<T> & r) 
  {
    assignFromMatrix(r);
  } 

  /** 
     Construct from a vector (x,y,z)+ angle where the Vector needs to implement x(), y() and z()
  */ 
  template<class Vector>
  AxisAngle(const Vector & v, const T & angle)  : 
    fAngle(angle) ,
    fAxis(v.unit() )
  {} 

  /** 
     Destructor (no operations)
  */ 
  ~AxisAngle() {}


  /** 
     Copy constructor
  */ 
  AxisAngle(const AxisAngle & r) : 
    fAngle(r.fAngle), 
    fAxis(r.fAxis)
  {}

  /** 
     Assignment operator
  */ 
  AxisAngle & operator = (const AxisAngle & r) 
  {
     if (this == &r) return *this;  // time saving self-test
     fAxis = r.fAxis;
     fAngle = r.fAngle;
     return *this;
  }

  /**
     accesss to rotation axis  
   */
  const Axis & axis() const { return fAxis; } 

  /**
     access to rotation angle
   */
  T angle() const { return fAngle; }

  /**
     Access axis angle object
  */
  const AxisAngle & axisAngle() const { 
    return *this; 
  }

  /**
     Access to 3x3 rotation matrix elements 
  */ 
  T operator() ( unsigned int i, unsigned int j) const { 
    // need to create 3D matrix (not very efficient) 
    Rotation3DMatrix<T> rot(*this);
    return rot(i,j);
  }

  /**
     Access to Euler rotation angles 
  */ 
  EulerAngles<T> eulerAngles() const { 
    Rotation3DMatrix<T> rot(*this);
    return EulerAngles<T>( rot ); 
  }

  /** 
      Access rotation matrix object
  */
  Rotation3DMatrix<T>  rotationMatrix() const { 
    return Rotation3DMatrix<T>(*this);
  }



protected: 

  /**
     find Axis & Angle from a rotation matrix
   */
  template<class RotMatrix>
  void assignFromMatrix(const RotMatrix & rot) { 
    // needs to be implemented
    fAngle = 0;
    fAxis = Axis(1,0,0);
  }


private: 

  T fAngle;
  Axis  fAxis;

}; 


  } // end namespace Math

} // end namespace ROOT


#endif /* MATHCORE_AXISANGLE */
