 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2005 , LCG ROOT MathLib Team                         *
  *                                                                    *
  *                                                                    *
  **********************************************************************/

// Header file for class Basic3DRotation
// 
// Created by: Lorenzo Moneta  at Wed May 11 11:25:53 2005
// 
// Last update: Wed May 11 11:25:53 2005
// 
#ifndef ROOT_MATH_BASIC3DROTATION
#define ROOT_MATH_BASIC3DROTATION 1

#include "GenVector/Rotation3DMatrix.h"
#include "GenVector/EulerAngles.h"
#include "GenVector/AxisAngle.h"
// needed for definition of size_t on gcc3.2 
#include <stdlib.h>


namespace ROOT { 

  namespace Math { 


  /**
     enumeration used to create rotation along X axis
   */
  enum RotationXAxis { RotationX }; 

  /**
     enumeration used to create rotation along X axis
   */
  enum RotationYAxis { RotationY }; 

  /**
     enumeration used to create rotation along X axis
   */
  enum RotationZAxis { RotationZ }; 



  /** 
     Generic Basic3DRotation class templated on the rotation type and the scalar type.
     The rotation type is the class which expresses the rotation. Possible RotationType are:  
     mathcore::Rotation3DMatrix , mathcore::EulerAngles and mathcore::AxisAngle 
     
  */ 
template<class RotationType >
class Basic3DRotation {



public: 

  typedef typename RotationType::Scalar T;
  typedef RotationType RotationCoordinateType;

  /** 
      Default constructor (identy rotation)
  */ 
  Basic3DRotation() {}
  


  /**
     Construct a rotation from Euler angles phi, theta and psi.
     See class mathcore::EulerAngles for angles definition
  */
  Basic3DRotation( const T & phi, const T & theta, const T & psi) : 
    fRotation(EulerAngles<T>(phi, theta, psi) )
  {}


  /**
     Construct a rotation from an axis and an angle. 
     The axis is expressed with a generic vector (does not need to be a unitary vector, but needs
     to implement the unit() method
  */
  template<class Vector> 
  Basic3DRotation( const Vector & axis, const T & angle) : 
    fRotation(AxisAngle<T>(axis, angle) )
  {}

  /**
     construct from any other type of rotation (using rotation matrix)
  */
  template<class ARotation> 
  explicit Basic3DRotation( const ARotation & r) : 
    fRotation( r.rotationMatrix() )
  {}

  /**
     Construct from a C array of size at least 9 copying the data  
   */ 
  explicit Basic3DRotation( const T * d) { 
    Rotation3DMatrix<T> rotMat(d);
    fRotation = rotMat; 
  }

  /**
     Construct from a linear algebra matrix of size at least 3x3 
  */
  // need to pass size to distringuish from other method
  template<class AMatrix>
  Basic3DRotation(const AMatrix & m, size_t size) { 
    Rotation3DMatrix<T> rotMat(m,size); 
    fRotation = rotMat;
  }

  /**
     Construct a rotation along X axis by an angle delta
   */
  Basic3DRotation( RotationXAxis, const T & delta ) { 
    DisplacementVector3D<Cartesian3D<T> > u(1,0,0);
    fRotation = AxisAngle<T>(u, delta);
  }

  /**
     Construct a rotation along Y axis
   */
  Basic3DRotation( RotationYAxis, const T & delta ) { 
    DisplacementVector3D<Cartesian3D<T> > u(0,1,0);
    fRotation = AxisAngle<T>(u, delta);
  }

  /**
     Construct a rotation along Z axis
   */
  explicit Basic3DRotation( RotationZAxis, const T & delta ) { 
    DisplacementVector3D<Cartesian3D<T> > u(0,0,1);
    fRotation = AxisAngle<T>(u, delta);
  }


  /** 
     Destructor (no operations)
  */ 
  ~Basic3DRotation() {}


  /** 
     Copy constructor
  */ 
  Basic3DRotation(const Basic3DRotation & r) : 
    fRotation(r.fRotation)
  {}

  /** 
     Assignment operator
  */ 
  Basic3DRotation & operator = (const Basic3DRotation & r) { 
    if (this == &r) return *this;
    fRotation = r.fRotation;
    return *this; 
  }

  /**
     Assign from any other type of rotation (using rotation matrix)
  */
  template<class ARotation> 
  Basic3DRotation & operator = ( const ARotation & r) { 
    fRotation = r.rotationMatrix(); 
    return *this;
  }

  /**
     access rotation matrix element
  */
  T operator() ( unsigned int i, unsigned int j) const { 
    return fRotation(i,j);
  }

  // get rotation elements
  T xx() const { return fRotation(0,0); }
  T xy() const { return fRotation(0,1); }
  T xz() const { return fRotation(0,2); }

  // get rotation elements
  T yx() const { return fRotation(1,0); }
  T yy() const { return fRotation(1,1); }
  T yz() const { return fRotation(1,2); }

  // get rotation elements
  T zx() const { return fRotation(2,0); }
  T zy() const { return fRotation(2,1); }
  T zz() const { return fRotation(2,2); }

  /**
     Phi Euler angle
  */
  T phi() const { return fRotation.eulerAngles().phi(); }

  /**
     Theta Euler angle
  */
  T theta() const { return fRotation.eulerAngles().theta(); }

  /**
     Psi Euler angle
  */
  T psi() const { return fRotation.eulerAngles().psi(); }

  /**
     return rotation matrix object
  */
  Rotation3DMatrix<T> rotationMatrix() const { 
    return fRotation.rotationMatrix();
  }

  /**
     return  EulerAngles object 
   */
  EulerAngles<T> eulerAngles() const { return  fRotation.eulerAngles(); }

  /**
     Rotation axis and angle
   */
  AxisAngle<T> axisAngle() const { return fRotation.axisAngle(); } 

  /**
     Get rotation axis
  */
  DisplacementVector3D<Cartesian3D<T>  > axis() const { return fRotation.axisAngle().axis(); }

  /**
     Get rotation angle
  */
  T delta() const { return fRotation.axisAngle().angle(); }


  /** 
      Access to a reference of the object reference representing the rotation (const version). 
      This method couls be used to get then the data (without copying) from the rotation matrix
  */ 
  const RotationType &  type() const {  return fRotation; }

  /** 
      Access to a reference of the object reference representing the rotation (non const version). 
      This method couls be used to get then the data (without copying) from the rotation matrix
  */ 
  RotationType & type() { return fRotation; }
  
  // operations

  /**
     Rotation operation on  a generic vector (has to be Gen3Dvector or LorentzVector)
   */
  template <class AVector>
  AVector operator() (const AVector & v) const { 
    const Rotation3DMatrix<T> & r = fRotation.rotationMatrix();
    return r.rotate(v);
  }

  /**
     Overload operator * for rotation on  a generic vector (has to be Vector3D or LorentzVector)
   */
  template <class AVector>
  AVector operator* (const AVector & v) const { 
    return this->operator()(v);
  }


  /**
     Multiply two rotations
   */
  template< class OtherRotType>
  Basic3DRotation & operator *= (const Basic3DRotation<OtherRotType> & r) { 
    // multiply the two matrixis (t.b.im.)
    return *this;
  }

  /**
     Multiply (combine) two rotations
   */
  template< class OtherRotType>
  Basic3DRotation  operator * (const  Basic3DRotation<OtherRotType> & r) { 
    Basic3DRotation tmp = *this; 
    tmp*= r;
    // multiply the two matrixis (t.b.im.)
    return tmp;
  }
  
  /**
      invert a rotation
   */
  Basic3DRotation & invert() {
    // t. b. impl.
    return *this;
  }

  /**
      return inverse of  a rotation
   */
  // move this as a global function ???
  Basic3DRotation inverse() {
    // t. b. impl.
    Basic3DRotation r = *this; 
    return r; 
  }

  /**
     rotation around X axis by angle delta
   */ 
  Basic3DRotation & rotateX( const T & delta) { 
    // t. b. i. 
    return *this; 
  }

  /**
     rotation around Y axis by angle delta
   */ 
  Basic3DRotation & rotateY( const T & delta) { 
    // t. b. i. 
    return *this; 
  }

  /**
     rotation around Z axis by angle delta
   */ 
  Basic3DRotation & rotateZ( const T & delta) { 
    // t. b. i. 
    return *this; 
  }

  /**
     rotation of axis to new axis specified by three orthonormal vectors. 
     The three vectors needs to be orthonormal and with a generic type which definex x(), y() and z() methods 
   */
  template<class AVector>
  Basic3DRotation & rotateAxes(const AVector & newX, const AVector & newY, const AVector & newZ) { 
    // t. b. i.
    return *this; 
  }

protected: 


  /**
     set of rotation type object 
   */
  void set(const  RotationType & r) { fRotation = r; } 



private: 

    RotationType fRotation;

}; 




  } // end namespace Math

} // end namespace ROOT


#endif /* MATHCORE_BASIC3DROTATION */
