/**********************************************************************
 *                                                                    *
 * Copyright (c) 2005 , LCG ROOT MathLib Team                         *
 *                                                                    *
 *                                                                    *
 **********************************************************************/

// Header file for class Basic3DTransformation
// 
// Created by: Lorenzo Moneta  at Wed May 18 17:39:38 2005
// 
// Last update: Wed May 18 17:39:38 2005
// 
#ifndef ROOT_MATH_BASIC3DTRANSFORMATION
#define ROOT_MATH_BASIC3DTRANSFORMATION 1


#include "GenVector/Basic3DRotation.h"
#include "GenVector/DisplacementVector3D.h"
#include "GenVector/PositionVector3D.h"



namespace ROOT { 

  namespace Math { 



  /** 
     Basic3DTransformation class describing  a rotation and then a translation
     The class is templated on the scalar type, rotation base type 
     (rotation matrix, Euler angles, etc..) and vector coordinate system type 
     (Cartesiaan, polar, etc..) representing the translation. 
     The internal data are a rotation data and a vector, but methods to access the data 
     like a 4x4 matrix are provided 

  */ 
  template <class RType, class VCType> 
  class Basic3DTransformation { 
    
    typedef Basic3DRotation<RType> Rotation; 
    typedef DisplacementVector3D<VCType> Translation; 

  public: 

    typedef typename RType::Scalar T;
    typedef RType RotationCoordinateType;
    typedef VCType VectorCoordinateType;



    /** 
	Default constructor (identy rotation) + zero translation
    */ 
    Basic3DTransformation() {}
    
    /**
       Construct from a rotation and a transformation
    */
    Basic3DTransformation( const Rotation & r, const Translation & v) : 
      fR( r ), fV(v )
    {}
    


    /**
       construct from any other type of transformation
    */
    template<class ATransform> 
    explicit Basic3DTransformation( const ATransform & t) : 
      fR( t.rotation() ), 
      fV(t.translation() )
    {}


    /** 
	Destructor (no operations)
    */ 
    ~Basic3DTransformation() {}
    

    /** 
	Copy constructor
    */ 
    Basic3DTransformation(const Basic3DTransformation & r) : 
      fR(r.fR), 
      fV(r.fV)
    {}

    /** 
	Assignment operator
    */ 
    Basic3DTransformation & operator = (const Basic3DTransformation & r) { 
      if (this == &r) return *this;
      fR = r.fR;
      fV = r.fV;
      return *this; 
    }

    /**
       Assign from any other type of transformation (using rotation matrix)
    */
    template<class ARotation> 
    Basic3DTransformation & operator = ( const ARotation & r) { 
      fR = r.rotation(); 
      fV = r.translation(); 
      return *this;
    }

  /**
     access transformation matrix element (4x4 matrix)
  */
    T operator() ( unsigned int i, unsigned int j) const { 
      if ( i < 3 && j < 3) 
	return fR(i,j);
      else if ( j == 3) { 
	if ( i == 0) return fV.x();
	if ( i == 1) return fV.y();
	if ( i == 2) return fV.z();
	if ( i == 3) return 1; 
      }
      return 0; 
    }

    // get rotation elements
    T xx() const { return fR(0,0); }
    T xy() const { return fR(0,1); }
    T xz() const { return fR(0,2); }
    
    // get rotation elements
    T yx() const { return fR(1,0); }
    T yy() const { return fR(1,1); }
    T yz() const { return fR(1,2); }
    
    // get rotation elements
    T zx() const { return fR(2,0); }
    T zy() const { return fR(2,1); }
    T zz() const { return fR(2,2); }
    
    // get translation elements
    T dx() const { return fV.x(); } 
    T dy() const { return fV.y(); } 
    T dz() const { return fV.z(); } 
    
    

    /**
       Get rotation representing the transformation
    */    
    Rotation rotation() { return fR;}

    /**
       Get the translation vector
    */
    Translation translation() { return fV; }


    /**
       Transformation operation on  points 
    */
    template<class CoordSystem2 > 
    PositionVector3D<CoordSystem2> operator() (const PositionVector3D <CoordSystem2> & p) const { 
      T vx = p.x();
      T vy = p.y(); 
      T vz = p.z();   
      Cartesian3D<T> c( xx()*vx + xy()*vy + xz()*vz + dx(),
			yx()*vx + yy()*vy + yz()*vz + dy(),
			zx()*vx + zy()*vy + zz()*vz + dz());
      return PositionVector3D<CoordSystem2>(c); 
    }

    /**
       specialized transformations for Cartesian points. 
    */

    PositionVector3D<Cartesian3D<T> > operator() (const PositionVector3D <Cartesian3D<T> > & p) const { 
      T vx = p.x();
      T vy = p.y(); 
      T vz = p.z();   
      return PositionVector3D<Cartesian3D<T> > ( xx()*vx + xy()*vy + xz()*vz + dx(),
					  yx()*vx + yy()*vy + yz()*vz + dy(),
					  zx()*vx + zy()*vy + zz()*vz + dz()  );
    }


    // also needs to implement on LV and * operator

    /**
       Combine this transformation with onother
    */ 
    template<class OtherRType, class OtherVCType  >
    Basic3DTransformation & operator *= (const Basic3DTransformation<OtherRType, OtherVCType> & t) { 
      // multiply the two matrixis (t.b.im.)
      return *this;
    }

    /**
       Combine two transformations, returning a new one
    */ 
    template<class OtherRType, class OtherVCType  >
    Basic3DTransformation  operator *  (const Basic3DTransformation<OtherRType, OtherVCType> & t) { 
      // multiply the two matrixis (t.b.im.)
      Basic3DTransformation tmp; 
      tmp *= t; 
      return tmp;
    }

  
    /**
       invert a transformation
    */
    Basic3DTransformation & invert() {
      // t. b. impl.
      return *this;
    }

    /**
       return inverse of  a transformation
    */
    // move this as a global function ???
    Basic3DTransformation inverse() {
      // t. b. impl.
      return Basic3DTransformation();
    }


  private: 
    Rotation fR; 
    Translation fV; 

  };


  // global functions 


  /**
     Multiply (combine) two transformation
  */
  template< class RType1, class VCType1,class RType2, class VCType2 >
  Basic3DTransformation<RType1, VCType1>  operator * (const Basic3DTransformation<RType1, VCType1> & t1, const Basic3DTransformation<RType2, VCType2> & t2 ) { 
    
    Basic3DTransformation<RType1, VCType1> tmp(t1); 
    // t. b. i.
    //tmp*= t2;
    // multiply the two matrixis (t.b.im.)
    return tmp;
  }




  } // end namespace Math

} // end namespace ROOT


#endif /* MATHCORE_BASIC3DTRANSFORMATION */
