 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2005 , LCG ROOT MathLib Team                         *
  *                                                                    *
  *                                                                    *
  **********************************************************************/

// Header file for class BoostMatrix
// 
// Created by: Lorenzo Moneta  at Thu May 12 11:26:05 2005
// 
// Last update: Thu May 12 11:26:05 2005
// 
#ifndef MATHCORE_BOOSTMATRIX
#define MATHCORE_BOOSTMATRIX 1

#include "GenVector/Rotation4DMatrix.h"
#include "GenVector/DisplacementVector3D.h"
#include <stdlib.h>



namespace ROOT { 

  namespace Math { 


  /** 
     BoostMatrix class reresenting Lorentz Boost as a 4x4 symmetric matrix (10 elements)  
     A Boost is represented by a direction vector and a magnitude beta 
  */ 
template <typename T> 
class BoostMatrix {

public: 

  typedef T Scalar;


  /**
     enumeration defining matrix elements index as stored in C array and size 
   */

  enum { Exx =  0, Exy =  1, Exz =  2, Ext =  3, 
	           Eyy =  4, Eyz =  5, Eyt =  6,
	                     Ezz =  7, Ezt =  8,
	                               Ett =  9,
	 EROT_DIM = 4, 
	 EROT_SIZE = 10
  };

  /** 
     Default constructor (rotaton is = I)
  */ 
  BoostMatrix() {
   for (unsigned int i = 0; i < EROT_DIM; ++i) { 
      for (unsigned int j = i; j < EROT_DIM; ++j) { 
	if ( i == j) 	
	  fData[fIndex(i,j)] = 1; 
	else 
	  fData[fIndex(i,j)] = 0; 
      }
   }
  } 

  /**
     Construct from a 3D vector representig the boost (beta = vector magnitude) 
   */
  template<typename, template <typename> class CoordSystem> 
  explicit BoostMatrix(const DisplacementVector3D<CoordSystem<T> > & v) { 
  }

  /**
     Contruct from three numbers representing betaX, betaY, betaZ
   */
  BoostMatrix(const T & betaX, const T & betaY, const T & betaZ) { 
  }

  /**
     Create from any 4x4 matrix or any object implementing operator (i,j)
     Matrix must be symmetric 
   */ 
  template <class Matrix> 
  explicit BoostMatrix( const Matrix & m, size_t size = EROT_SIZE) {
    assert (size >= EROT_SIZE);
    for (unsigned int i = 0; i < EROT_DIM; ++i) { 
      for (unsigned int j = i; j < EROT_DIM; ++j) { 
	fData[fIndex(i,j)] = m(i,j); 
      }
    }
  }

  /**
     Create from a C array of size at least size 10 
   */ 
  explicit BoostMatrix( const T * d) { 
    memcpy(fData, d, EROT_SIZE*sizeof(T) );
  }


  /** 
     Destructor (no operations)
  */ 
  ~BoostMatrix() {}


  /** 
     Copy constructor
  */ 
  BoostMatrix(const BoostMatrix & r) { 
    memcpy(fData, r.fData, EROT_SIZE*sizeof(T) );
  } 


  /** 
     Assignment operator
  */ 
  BoostMatrix & operator = (const BoostMatrix & r) { 
    if (this == &r)  return *this;
    memcpy(fData, r.fData, EROT_SIZE*sizeof(T) );
    return *this;
  }

  /**
     Assign from any 4x4 symmetric matrix or any object implementing operator (i,j)   
   */ 
  template <class Matrix> 
  BoostMatrix & operator = ( const Matrix & m) { 
    for (unsigned int i = 0; i < EROT_DIM; ++i) { 
      for (unsigned int j = i; j < EROT_DIM; ++j) { 
	fData[fIndex(i,j)] = m(i,j); 
      }
    }
    return *this;
  }


  /**
     Assign from a C array of size at least 10  
   */ 
  BoostMatrix & operator = ( const T * d) { 
    memcpy(fData, d, EROT_SIZE*sizeof(T) );
    return *this;
  }


  /** 
      Access rotation matrix object
  */
  const BoostMatrix & boost() const { 
    return *this; 
  }


  /**
     Access to matrix elements 
   */ 
  T operator() ( unsigned int i, unsigned int j) const { 
    return (fData[fIndex(i,j)]);
  }

  /**
     Return C array (10 elements) representing the rotation (non const version)
  */
  T * data() { return fData; }

  /**
     Return C array (10 elements) representing the rotation (const version)
  */
  const T * data() const { return fData; }

   /**
      return vector representing the boost 
   */
  DisplacementVector3D<Cartesian3D<T> >  boostVector() const { 
    // t. b. impl.
    return  DisplacementVector3D<Cartesian3D<T> >();
  }

   /**
      return vector representing boost direction 
   */
  DisplacementVector3D<Cartesian3D<T> >  boostDirection() const { 
    // t. b. impl.
    return  DisplacementVector3D<Cartesian3D<T> >();
  }

   /**
      return beta of the  boost  
   */
  T  beta() const { 
    // t. b. impl.
    return  0;
  }

  T gamma() const { 
    // t. b. im.
    return 0; 
  }



  /**
     rotation (boost) of a generic Lorentz  vector (4D)
  */
  template <class CoordSystem>
  BasicLorentzVector<CoordSystem> rotate (BasicLorentzVector<CoordSystem>  const  & v) const { 
    // make the operation using the vector type precision (t. b. checked)
    typedef typename CoordSystem::Scalar T2;
    register T2 x = v.x();
    register T2 y = v.y();
    register T2 z = v.z();
    register T2 t = v.t();
    return BasicLorentzVector<CoordSystem> ( 
						Cartesian4D<T2> (
						fData[Exx]*x + fData[Exy]*y + fData[Exz]*z + fData[Ext]*t,
						fData[Exy]*x + fData[Eyy]*y + fData[Eyz]*z + fData[Eyt]*t,
						fData[Exz]*x + fData[Eyz]*y + fData[Ezz]*z + fData[Ezt]*t, 
						fData[Ext]*x + fData[Eyt]*y + fData[Ezt]*z + fData[Ett]*t 
						) );
  }

 


protected: 
  /**
     get single index from i,j
   */
  unsigned int fIndex(unsigned int i, unsigned int j) const { 
    assert( i < EROT_DIM );
    assert( j < EROT_DIM );
    if ( i > j) 
      return j + i*(i+1)/2;
    else 
      return i + j*(j+1)/2;
  }

  

private: 

  T fData[EROT_SIZE];
  


}; 



  } // end namespace Math

} // end namespace ROOT


#endif /* MATHCORE_BOOSTMATRIX */
