 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2005 , LCG ROOT MathLib Team                         *
  *                                                                    *
  *                                                                    *
  **********************************************************************/

// Header file for class EulerAngles
// 
// Created by: Lorenzo Moneta  at Tue May 10 17:55:10 2005
// 
// Last update: Tue May 10 17:55:10 2005
// 
#ifndef ROOT_MATH_EULERANGLES
#define ROOT_MATH_EULERANGLES 1

#include "GenVector/Rotation3DMatrix.h"
#include "GenVector/AxisAngle.h"



namespace ROOT { 

  namespace Math { 


  /** 
     EulerAngles class describing rotation as three angles (Euler Angles). 
     The Euler angles definition matches that of Classical Mechanics (Goldstein). 
     See also <A HREF="http://mathworld.wolfram.com/EulerAngles.html">here</A> for Euler angles definition.      
  */ 
template<typename T>
class EulerAngles {

public: 

  typedef T Scalar;


  /** 
     Default constructor
  */ 
  EulerAngles() : 
    fPhi(0), fTheta(0), fPsi(0)
  {}

  /** 
     Constructor from phi, theta and psi
  */ 
  EulerAngles( const T & phi, const T & theta, const T & psi ) : 
    fPhi(phi), fTheta(theta), fPsi(psi)
  {}

  /** 
     Construct from a rotation matrix
  */ 
  EulerAngles(const Rotation3DMatrix<T> & r) 
  {
    assignFromMatrix( r );
  } 


  /** 
     Destructor (no operations)
  */ 
  ~EulerAngles() {}

  /** 
     Copy constructor
  */ 
  EulerAngles(const EulerAngles & e) { 
    fPhi = e.fPhi;
    fTheta = e.fTheta;
    fPsi = e.fPsi;
  }

  /** 
     Assignment operator
  */ 
  EulerAngles & operator = (const EulerAngles & e) { 
    if ( this == &e) return *this;
    fPhi = e.fPhi;
    fTheta = e.fTheta;
    fPsi = e.fPsi;
    return *this;
  }


  /** 
     Assign from a rotation matrix
  */ 
  EulerAngles & operator = (const Rotation3DMatrix<T> & r) { 
    assignFromMatrix( r );
    return *this;
  }

  /** 
      Return Phi Euler angle
  */
  T phi() const { return fPhi; } 

  /** 
      Return Theta Euler angle
  */
  T theta() const { return fTheta; } 

  /** 
      Return Psi Euler angle
  */
  T psi() const { return fPsi; } 

  /** 
      Access Euler Angle object
  */
  const EulerAngles & eulerAngles() const { 
    return *this;
  }


  /**
     Access 3x3 rotation matrix element
   */
  T operator() ( unsigned int i, unsigned int j) const { 
    // need to create 3D matrix (not very efficient) 
    Rotation3DMatrix<T> rot(*this);
    return rot(i,j);
  }


  /**
     Access to rotation axis and angle
  */
  AxisAngle<T> axisAngle() const {
    Rotation3DMatrix<T> rot(*this);
    return AxisAngle<T>(rot);
  }

  /** 
      Access rotation matrix object
  */
  Rotation3DMatrix<T>  rotationMatrix() const { 
    return Rotation3DMatrix<T>(*this);
  }


protected: 

  /**
     find Euler Angles from a 3x3 rotation matrix
   */
  template<class RotMatrix>
  void assignFromMatrix(const RotMatrix & rot) { 
    // needs to be implemented
    fPhi = 0; 
    fTheta = 0; 
    fPsi = 0;
  }

private: 

  T fPhi;
  T fTheta; 
  T fPsi; 

}; 


  } // end namespace Math

} // end namespace ROOT


#endif /* MATHCORE_EULERANGLES */
