 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2005 , LCG ROOT MathLib Team                         *
  *                                                                    *
  *                                                                    *
  **********************************************************************/

// Header file for class Rotation3DMatrix
// 
// Created by: Lorenzo Moneta  at Tue May 10 14:57:05 2005
// 
// Last update: Tue May 10 14:57:05 2005
// 
#ifndef MATHCORE_ROTATION3DMATRIX
#define MATHCORE_ROTATION3DMATRIX 1

#include "GenVector/AxisAngle.h"
#include "GenVector/EulerAngles.h"
#include "GenVector/PositionVector3D.h"
#include "GenVector/DisplacementVector3D.h"
#include "GenVector/BasicLorentzVector.h"


#include <assert.h>
#include <stdlib.h>



namespace ROOT { 

  namespace Math { 


  /** 
     Rotation3DMatrix class describing rotation as a 3x3 matrix based on cartesian axis
  */ 

template <typename T>
class Rotation3DMatrix {

public: 

  typedef T Scalar;


  /**
     enumeration defining matrix elements index as stored in C array and size 
   */

  enum { Exx = 0, Exy = 1, Exz = 2, 
	 Eyx = 3, Eyy = 4, Eyz = 5, 
	 Ezx = 6, Ezy = 7, Ezz = 8, 
	 EROT_DIM = 3, 
	 EROT_SIZE = 9
  };

  /** 
     Default constructor (rotaton is = I)
  */ 
  Rotation3DMatrix() {
   for (unsigned int i = 0; i < 3; ++i) { 
      for (unsigned int j = 0; j < 3; ++j) { 
	if ( i == j) 	
	  fData[fIndex(i,j)] = 1; 
	else 
	  fData[fIndex(i,j)] = 0; 
      }
   }
  } 

  /**
     Create from any 3x3 matrix or any object implementing operator (i,j)   
   */ 
  template <class Matrix> 
  explicit Rotation3DMatrix( const Matrix & m, size_t size = 9) {
    assert (size >= EROT_SIZE);
    for (unsigned int i = 0; i < EROT_DIM; ++i) { 
      for (unsigned int j = 0; j < EROT_DIM; ++j) { 
	fData[fIndex(i,j)] = m(i,j); 
      }
    }
  }

  /**
     Create from a C array of size at least 9  
   */ 
  explicit Rotation3DMatrix( const T * d) { 
    memcpy(fData, d, EROT_SIZE*sizeof(T) );
  }

  /**
     Create from direction axis and angle
   */
  Rotation3DMatrix( const AxisAngle<T> & a) { 
    setAxisAngle(a);
  }

  /**
     Create from Euler angles
   */
  Rotation3DMatrix( const EulerAngles<T> & e) { 
    setEulerAngles(e);
  }


  /** 
     Destructor (no operations)
  */ 
  ~Rotation3DMatrix() {}


  /** 
     Copy constructor
  */ 
  Rotation3DMatrix(const Rotation3DMatrix & r) { 
    memcpy(fData, r.fData, 9*sizeof(T) );
  } 


  /** 
     Assignment operator
  */ 
  Rotation3DMatrix & operator = (const Rotation3DMatrix & r) { 
    if (this == &r)  return *this;
    memcpy(fData, r.fData, 9*sizeof(T) );
    return *this;
  }

  /**
     Assign from any 3x3 matrix or any object implementing operator (i,j)   
   */ 
  template <class Matrix> 
  Rotation3DMatrix & operator = ( const Matrix & m) { 
    for (unsigned int i = 0; i < 3; ++i) { 
      for (unsigned int j = 0; j < 3; ++j) { 
	fData[fIndex(i,j)] = m(i,j); 
      }
    }
    return *this;
  }


  /**
     Assign from a C array of size at least 9  
   */ 
  Rotation3DMatrix & operator = ( const T * d) { 
    memcpy(fData, d, 9*sizeof(T) );
    return *this;
  }


  /**
     Assign from direction axis and angle
   */
  Rotation3DMatrix & operator = ( const AxisAngle<T> & a) { 
    setAxisAngle(a);
    return *this;
  }

  /**
     Assign from Euler angles
   */
  Rotation3DMatrix & operator = ( const EulerAngles<T> & e) { 
    setEulerAngles(e);
    return *this;
  }

  /** 
      Access rotation matrix object
  */
  const Rotation3DMatrix & rotationMatrix() const { 
    return *this; 
  }


  /**
     Access to matrix elements 
   */ 
  T operator() ( unsigned int i, unsigned int j) const { 
    return (fData[fIndex(i,j)]);
  }

  /**
     Return C array (9 elements) representing the rotation (non const version)
  */
  T * data() { return fData; }

  /**
     Return C array (9 elements) representing the rotation (const version)
  */
  const T * data() const { return fData; }

  /**
     Access to Euler Angles 
  */
  EulerAngles<T> eulerAngles() const { 
    return EulerAngles<T>(*this); 
  }

  /**
     Access to axis and angle
  */
  AxisAngle<T> axisAngle() const { 
    return AxisAngle<T>(*this);
  }

  /**
     rotation of af 3D vector 
  */
  template <class CoordSystem>
  DisplacementVector3D<CoordSystem> rotate (DisplacementVector3D<CoordSystem>  const  & v) const { 
    // make the operation using the vector type precision (t. b. checked)
    typedef typename CoordSystem::Scalar T2;
    register T2 x = v.x();
    register T2 y = v.y();
    register T2 z = v.z();
    return DisplacementVector3D<CoordSystem> (
					 Cartesian3D<T2> ( 
					 fData[Exx]*x + fData[Exy]*y + fData[Exz]*z,
					 fData[Eyx]*x + fData[Eyy]*y + fData[Eyz]*z,
					 fData[Ezx]*x + fData[Ezy]*y + fData[Ezz]*z
					 ) );
  }


  /**
     rotation of af 3D point 
  */
  template <class CoordSystem>
  PositionVector3D<CoordSystem> rotate (PositionVector3D<CoordSystem>  const  & v) const { 
    // make the operation using the vector type precision (t. b. checked)
    typedef typename CoordSystem::Scalar T2;
    register T2 x = v.x();
    register T2 y = v.y();
    register T2 z = v.z();
    return PositionVector3D<CoordSystem> (
					 Cartesian3D<T2> ( 
					 fData[Exx]*x + fData[Exy]*y + fData[Exz]*z,
					 fData[Eyx]*x + fData[Eyy]*y + fData[Eyz]*z,
					 fData[Ezx]*x + fData[Ezy]*y + fData[Ezz]*z
					 ) );
  }

  // is really needed ? put a dependency on LV classes from 3D ? 

  /**
     rotation of a generic Lorentz  vector (4D)
  */
  template <class CoordSystem>
  BasicLorentzVector<CoordSystem> rotate (BasicLorentzVector<CoordSystem>  const  & v) const { 
    // make the operation using the vector type precision (t. b. checked)
    typedef typename CoordSystem::Scalar T2;
    register T2 x = v.x();
    register T2 y = v.y();
    register T2 z = v.z();
    return BasicLorentzVector<CoordSystem> ( 
						Cartesian4D<T2> (  
						fData[Exx]*x + fData[Exy]*y + fData[Exz]*z,
						fData[Eyx]*x + fData[Eyy]*y + fData[Eyz]*z,
						fData[Ezx]*x + fData[Ezy]*y + fData[Ezz]*z, 
						v.t() ) );
  }


protected: 
  /**
     get single index from i,j
   */
  unsigned int fIndex(unsigned int i, unsigned int j) const { 
    assert( i < EROT_DIM );
    assert( j < EROT_DIM );
    return i*EROT_DIM + j;
  }


  /**
     set rotation matrix from Euler Angles 
  */
  void setEulerAngles(const EulerAngles<T> & e) { 

    T phi = e.phi();
    T theta = e.theta();
    T psi = e.psi();

    register T sinPhi   = sin( phi   ), cosPhi   = cos( phi   );
    register T sinTheta = sin( theta ), cosTheta = cos( theta );
    register T sinPsi   = sin( psi   ), cosPsi   = cos( psi   );

    fData[Exx] =   cosPsi * cosPhi - cosTheta * sinPhi * sinPsi;
    fData[Exy] =   cosPsi * sinPhi + cosTheta * cosPhi * sinPsi;
    fData[Exz] =   sinPsi * sinTheta;
    
    fData[Eyx] = - sinPsi * cosPhi - cosTheta * sinPhi * cosPsi;
    fData[Eyy] = - sinPsi * sinPhi + cosTheta * cosPhi * cosPsi;
    fData[Eyz] =   cosPsi * sinTheta;

    fData[Ezx] =   sinTheta * sinPhi;
    fData[Ezy] = - sinTheta * cosPhi;
    fData[Ezz] =   cosTheta;
  }

  /**
     set rotation matrix from rotation axis + angle
  */
  void setAxisAngle( const AxisAngle<T> & a) { 
    
    T delta = a.angle();
    DisplacementVector3D<Cartesian3D<T> > u = a.axis();

    register T sinDelta = sin(delta), cosDelta = cos(delta);
    register T oneMinusCosDelta = 1.0 - cosDelta;
    
    
    register T uX = u.x();
    register T uY = u.y();
    register T uZ = u.z();
      
    fData[Exx] = oneMinusCosDelta * uX * uX  +  cosDelta;
    fData[Exy] = oneMinusCosDelta * uX * uY  -  sinDelta * uZ;
    fData[Exz] = oneMinusCosDelta * uX * uZ  +  sinDelta * uY;
    
    fData[Eyx] = oneMinusCosDelta * uY * uX  +  sinDelta * uZ;
    fData[Eyy] = oneMinusCosDelta * uY * uY  +  cosDelta;
    fData[Eyz] = oneMinusCosDelta * uY * uZ  -  sinDelta * uX;
      
    fData[Ezx] = oneMinusCosDelta * uZ * uX  -  sinDelta * uY;
    fData[Ezy] = oneMinusCosDelta * uZ * uY  +  sinDelta * uX;
    fData[Ezz] = oneMinusCosDelta * uZ * uZ  +  cosDelta;

  }
  

private: 

  T fData[EROT_SIZE];
  

}; 


  } // end namespace Math

} // end namespace ROOT


#endif /* MATHCORE_ROTATION3DMATRIX */
