 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2005 , LCG ROOT MathLib Team                         *
  *                                                                    *
  *                                                                    *
  **********************************************************************/

// Header file for class Rotation4DMatrix
// 
// Created by: Lorenzo Moneta  at Wed May 11 17:48:56 2005
// 
// Last update: Wed May 11 17:48:56 2005
// 
#ifndef MATHCORE_ROTATION4DMATRIX
#define MATHCORE_ROTATION4DMATRIX 1

#include "GenVector/BasicLorentzVector.h"

#include <stdlib.h>



namespace ROOT { 

  namespace Math { 



  /** 
     Rotation4DMatrix class for describing LorentzRotations
  */ 
template <typename T>
class Rotation4DMatrix {

public: 

  typedef T Scalar;


  /**
     enumeration defining matrix elements index as stored in C array and size 
   */

  enum { Exx =  0, Exy =  1, Exz =  2, Ext =  3, 
	 Eyx =  4, Eyy =  5, Eyz =  6, Eyt =  7,
	 Ezx =  8, Ezy =  9, Ezz = 10, Ezt = 11,
	 Etx = 12, Ety = 13, Etz = 14, Ett = 15,
	 EROT_DIM = 4, 
	 EROT_SIZE = 16
  };

  /** 
     Default constructor (rotaton is = I)
  */ 
  Rotation4DMatrix() {
   for (unsigned int i = 0; i < EROT_DIM; ++i) { 
      for (unsigned int j = 0; j < EROT_DIM; ++j) { 
	if ( i == j) 	
	  fData[fIndex(i,j)] = 1; 
	else 
	  fData[fIndex(i,j)] = 0; 
      }
   }
  } 

  /**
     Create from any 4x4 matrix or any object implementing operator (i,j)   
   */ 
  template <class Matrix> 
  explicit Rotation4DMatrix( const Matrix & m, size_t size = EROT_SIZE) {
    assert (size >= EROT_SIZE);
    for (unsigned int i = 0; i < EROT_DIM; ++i) { 
      for (unsigned int j = 0; j < EROT_DIM; ++j) { 
	fData[fIndex(i,j)] = m(i,j); 
      }
    }
  }

  /**
     Create from a C array of size at least 16
   */ 
  explicit Rotation4DMatrix( const T * d) { 
    memcpy(fData, d, EROT_SIZE*sizeof(T) );
  }


  /** 
     Destructor (no operations)
  */ 
  ~Rotation4DMatrix() {}


  /** 
     Copy constructor
  */ 
  Rotation4DMatrix(const Rotation4DMatrix & r) { 
    memcpy(fData, r.fData, EROT_SIZE*sizeof(T) );
  } 


  /** 
     Assignment operator
  */ 
  Rotation4DMatrix & operator = (const Rotation4DMatrix & r) { 
    if (this == &r)  return *this;
    memcpy(fData, r.fData, EROT_SIZE*sizeof(T) );
    return *this;
  }

  /**
     Assign from any 4x4 matrix or any object implementing operator (i,j)   
   */ 
  template <class Matrix> 
  Rotation4DMatrix & operator = ( const Matrix & m) { 
    for (unsigned int i = 0; i < EROT_DIM; ++i) { 
      for (unsigned int j = 0; j < EROT_DIM; ++j) { 
	fData[fIndex(i,j)] = m(i,j); 
      }
    }
    return *this;
  }


  /**
     Assign from a C array of size at least 9  
   */ 
  Rotation4DMatrix & operator = ( const T * d) { 
    memcpy(fData, d, EROT_SIZE*sizeof(T) );
    return *this;
  }


  /** 
      Access rotation matrix object
  */
  const Rotation4DMatrix & rotationMatrix() const { 
    return *this; 
  }


  /**
     Access to matrix elements 
   */ 
  T operator() ( unsigned int i, unsigned int j) const { 
    return (fData[fIndex(i,j)]);
  }

  /**
     Return C array (16 elements) representing the rotation (non const version)
  */
  T * data() { return fData; }

  /**
     Return C array (16 elements) representing the rotation (const version)
  */
  const T * data() const { return fData; }

//   /**
//      Access to boost 
//   */
//   Boost<T> boost() { 
//     return Boost<T>(*this);
//   }


  /**
     rotation of a generic Lorentz  vector (4D)
  */
  template <class CoordSystem>
  BasicLorentzVector<CoordSystem> rotate (BasicLorentzVector<CoordSystem>  const  & v) const { 
    // make the operation using the vector type precision (t. b. checked)
    typedef typename CoordSystem::Scalar T2;
    register T2 x = v.x();
    register T2 y = v.y();
    register T2 z = v.z();
    register T2 t = v.t();
    return BasicLorentzVector<CoordSystem> ( 
						Cartesian4D<T2> (
						fData[Exx]*x + fData[Exy]*y + fData[Exz]*z + fData[Ext]*t,
						fData[Eyx]*x + fData[Eyy]*y + fData[Eyz]*z + fData[Eyt]*t,
						fData[Ezx]*x + fData[Ezy]*y + fData[Ezz]*z + fData[Ezt]*t, 
						fData[Etx]*x + fData[Ety]*y + fData[Etz]*z + fData[Ett]*t 
						) );
  }


protected: 
  /**
     get single index from i,j
   */
  unsigned int fIndex(unsigned int i, unsigned int j) const { 
    assert( i < EROT_DIM );
    assert( j < EROT_DIM );
    return i*EROT_DIM + j;
  }

  

private: 

  T fData[EROT_SIZE];
  

}; 


  } // end namespace Math

} // end namespace ROOT


#endif /* MATHCORE_ROTATION4DMATRIX */
